# 一个自定义异常类的模块

class GameUnitError(Exception):
    def __init__(self, message=''):
        self.padding = '~' * 50 + '\n'
        # 未指定错误
        self.error_message = 'Unspecified Error'
    # print(self.error_message)


class HealthMeterException(GameUnitError):
    def __init__(self, message=''):
        super(HealthMeterException, self).__init__(message)
        self.error_message = (self.padding + 'ERROR:Health Meter Problem' + '\n' + self.padding)


class HutError(GameUnitError):
    def __init__(self, message='', code=000):
        super().__init__(message)
        self.error_message = '~' * 50 + '\n'
        self.error_dict = {
            000: "ERROR-000: Unspecified Error!",
            101: "ERROR-101: Health Meter Problem!",
            102: "ERROR-102: Attack issue! Ignored",
        }
        try:
            self.error_message += self.error_dict[code]
        except KeyError:
            self.error_message += self.error_dict[000]
        self.error_message += '\n' + '~' * 50



from abc import ABCMeta, abstractmethod
from gameuniterror import HealthMeterException
from gameutils import *


class AbstractGameUnit(metaclass=ABCMeta):

    def __init__(self):
        self.name = None
        self.type = None
        self.max_hp = 0
        self.health_meter = 0
        self.unit_type = None

    @abstractmethod
    def _info(self):
        print(f'我是一个{self.name}')

    def heal(self, heal_by=10, full_healing=False):
        if self.health_meter == self.max_hp:
            print('你已满血')
            return

        if full_healing:
            self.health_meter = self.max_hp
            print('加满了')
        else:
            self.health_meter += heal_by
            print("好队友")
        if self.health_meter > self.max_hp:
            raise HealthMeterException('血量大于最大生命值了!')
        # max = a if a>b else b

    def reset_health_meter(self):
        """Reset the `health_meter` (assign default hit points)"""
        self.health_meter = self.max_hp

    def show_health(self, bold=True, end='\n'):

        msg = "Health: %s: %d" % (self.name, self.health_meter)

        if bold:
            print_bold(msg, end=end)
        else:
            print(msg, end=end)



from abstractgameunit import AbstractGameUnit


class OrcRider(AbstractGameUnit):
    def __init__(self):
        super().__init__()
        self.name = '兽人'
        self.max_hp = 30
        self.health_meter = 30
        self.unit_type = '敌军'

    def _info(self):
        print('很傻很天真')



import random

from gameutils import *
from hut import Hut
from knight import Knight
from orcider import OrcRider


# 改写为 abstractClass


class AttackofTheOrcs:
    def __init__(self):
        self.huts = []
        self.player = None
        self.is_acquired_list = ['未知'] * 5

    @staticmethod
    def show_game_mission():
        print("阴阳二神互相杀伐，世界处于永劫之中。玩家需扮演一名英雄，在地图中收集各种资源，与其他玩家对抗，生存到最后。")

    def create_occupy_huts(self):
        # friend = Knight()
        # enemy = OrcRider()
        choice_list = ['友军', '敌军', None]
        for i in range(5):
            computer_choice = random.choice(choice_list)
            if computer_choice == '敌军':
                self.huts.append(Hut(i + 1, OrcRider()))
            elif computer_choice == '友军':

                self.huts.append(Hut(i + 1, Knight()))
            else:
                self.huts.append(Hut(i + 1, computer_choice))
            # print(self.huts[i])

    def check_occupant_all_acquired_enemy_death(self):
        a = 0
        e = 0
        z = 0
        for i in range(len(self.huts)):
            if self.huts[i].is_acquired:
                a += 1
                if self.huts[i].occupant is not None:
                    if self.huts[i].occupant.unit_type == '敌军':
                        e += 1
                        if self.huts[i].occupant.health_meter == 0:
                            print(self.huts[i].occupant.health_meter)
                            z += 1

        if a + z > len(self.huts) + e - 1:
            return True
        else:
            return False

    @staticmethod
    def user_choice():
        while True:
            try:
                idx = int(input('\n选择你前往的点...1~5 ') or 2)

            except ValueError:
                print('请输入数字')
                continue
            else:
                return idx

    #     raise AssertionError('随手报错')

    def enter_huts(self):

        all_is_acquired = self.check_occupant_all_acquired_enemy_death()

        self.is_acquired_list = ['未知'] * 5
        # print(self.is_acquired_list)

        while all_is_acquired is False:
            print(self.is_acquired_list)
            idx = self.user_choice()
            try:
                self.huts[idx - 1].is_acquired
            except IndexError:
                print('你输入的值太大了')
                continue

            else:

                if self.huts[idx - 1].is_acquired:

                    print('我来过这个地方')
                    if self.huts[idx - 1].occupant is not None and self.huts[idx - 1].occupant.unit_type == '敌军':
                        if self.huts[idx - 1].occupant.health_meter > 0:
                            self.player.attack(self.huts[idx - 1].occupant)
                elif self.huts[idx - 1].occupant is None:
                    print('此处空无一物！')
                    self.huts[idx - 1].is_acquired = True
                    self.is_acquired_list[idx - 1] = '空'
                else:
                    # idx = self.user_choice()
                    # print(self.huts[idx-1].occupant.name)
                    if self.huts[idx - 1].occupant.unit_type == '敌军':
                        print('是敌人')
                        # (self.huts[idx-1].__dict__.items())
                        self.player.attack(self.huts[idx - 1].occupant)
                        self.is_acquired_list[
                            idx - 1] = f"{self.huts[idx - 1].occupant.name}:{self.huts[idx - 1].occupant.health_meter}"
                        # self.is_acquired_list[idx - 1] = self.huts[idx - 1].occupant.name
                        self.huts[idx - 1].is_acquired = True
                        # all_is_acquired = self.check_occupant_is_acquired()
                    if self.player.health_meter <= 0:
                        print('没血了，重开！')
                        break
                    # self.huts[idx - 1].is_acquired = True

                    elif self.huts[idx - 1].occupant.unit_type == '友军':

                        print('是友军')
                        self.player.heal()

                        self.huts[idx - 1].is_acquired = True
                        self.is_acquired_list[idx - 1] = self.huts[idx - 1].occupant.name
                all_is_acquired = self.check_occupant_all_acquired_enemy_death()
                if self.player.health_meter <= 0:
                    print('没血了，重开！')
                    break
            finally:
                pass
                # print('finally')
        print(self.is_acquired_list)
        if all_is_acquired is True:
            self.player.show_health()
            print_bold('战斗结束,不朽面具！')
        # return idx

    # def

    def play(self):
        self.player = Knight()
        self.show_game_mission()
        self.create_occupy_huts()
        self.enter_huts()
        # self.
        # print(self.huts)
    # def OrcRider(self):
    #     pass


if __name__ == '__main__':
    game = AttackofTheOrcs()
    game.play()



from distutils.core import setup

with open('README') as file:
    readme = file.read()


setup(
    name='chys_private',
    version='2.0.0',
    packages=['wargame'],
    url='http://localhost:8071/simple',
    description='my private game',
    long_description=readme,
    author='luoquan',
    author_email='178517202@qq.com'
)



from abstractgameunit import AbstractGameUnit
from gameutils import *
import random
class Knight(AbstractGameUnit):
    def __init__(self, name='权哥'):
        super(Knight, self).__init__()
        super().__init__()
        self.name = name
        self.max_hp = 100
        self.health_meter = 100
        self.unit_type = '友军'

    def _info(self):
        print(f'断剑重铸之日，骑士归来之时')

    def attack(self, enemy):
        # print('开始打架')
        # continue_attack = 'y'
        while (self.health_meter * enemy.health_meter) > 0:

            continue_attack = input('开始攻击？（y/n）:' or 'y')

            if continue_attack == 'n':

                print('跑路了')
                break

            elif continue_attack == 'y':
                injured_unit = random.choice([self, enemy])
                injury = random.randint(10, 15)
                print_bold(f'{self.name}血量为：{self.health_meter}{enemy.name}血量为：{enemy.health_meter}')
                injured_unit.health_meter = max(injured_unit.health_meter - injury, 0)
                print(f'开始战斗,{injured_unit.name}受到{injury}点伤害,血量是{injured_unit.health_meter}！')
            # else:
            #     print('请输入y/n')

            if self.health_meter <= 0:
                print('你没血了')
                break
            elif enemy.health_meter <= 0:
                print('你干掉了对方')
                break
                # self.attack(self,enemy)
